import argparse
from blueness import module
from blueness.argparse.generic import sys_exit
from blue_options import NAME
from blue_options.env import CYAN, NC
from blue_options.logger.config import logger

NAME = module.name(__file__, NAME)


parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    default="",
    help="hr",
)
parser.add_argument(
    "--width",
    type=int,
    default=80,
)
parser.add_argument(
    "--pattern",
    type=str,
    default=". .. ... .. ",
)
args = parser.parse_args()


success = False
if args.task == "hr":
    success = True
    print(
        "{}{}{}".format(
            CYAN,
            ((args.width // len(args.pattern) + 1) * args.pattern)[: args.width],
            NC,
        )
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



import argparse
from blueness import module
from blueness.argparse.generic import sys_exit
from blue_options import NAME
from blue_options.env import GREEN, NC
from blue_options.logger.config import logger

NAME = module.name(__file__, NAME)

LIST_OF_TASKS = "in|intersect|item|len|log|nonempty|resize|sort"

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help=LIST_OF_TASKS,
)
parser.add_argument(
    "--after",
    type=str,
    default="item(s)",
)
parser.add_argument(
    "--before",
    type=str,
    default="list of",
)
parser.add_argument(
    "--count",
    type=int,
)
parser.add_argument(
    "--delim",
    type=str,
    default=",",
)
parser.add_argument(
    "--index",
    type=int,
)
parser.add_argument(
    "--item",
    type=str,
)
parser.add_argument(
    "--items",
    type=str,
)
parser.add_argument(
    "--items_1",
    type=str,
)
parser.add_argument(
    "--items_2",
    type=str,
)
parser.add_argument(
    "--unique",
    type=int,
    help="0|1",
    default=0,
)
args = parser.parse_args()

delim = " " if args.delim == "space" else args.delim

list_of_items = (
    [item for item in args.items.split(delim) if item]
    if isinstance(args.items, str)
    else None
)

success = args.task in LIST_OF_TASKS.split("|")
if args.task == "in":
    print("True" if args.item in list_of_items else "False")
elif args.task == "intersect":
    print(
        delim.join(
            [
                item
                for item in args.items_1.split(delim)
                if item in args.items_2.split(delim) and item
            ]
        )
    )
elif args.task == "item":
    print(list_of_items[args.index])
elif args.task == "len":
    print(len(list_of_items))
elif args.task == "log":
    print(
        "{} {} {}: {}".format(
            args.before,
            f"{GREEN}{len(list_of_items)}{NC}",
            args.after,
            "{}{}{}.".format(
                GREEN,
                f"{NC}, {GREEN}".join(list_of_items),
                NC,
            ),
        )
    )
elif args.task == "nonempty":
    print(delim.join(list_of_items))
elif args.task == "resize":
    print(delim.join(list_of_items[: args.count]))
elif args.task == "sort":
    if args.unique:
        list_of_items = list(set(list_of_items))

    print(delim.join(sorted(list_of_items)))
else:
    success = None

sys_exit(logger, NAME, args.task, success)



import time
from blue_options import string


class ElapsedTimer:
    def __init__(self):
        self.start_time = time.time()
        self.elapsed_time = None

    def stop(self):
        if self.start_time is None:
            return

        self.elapsed_time = time.time() - self.start_time
        self.start_time = None

    def elapsed_pretty(self, **kwargs):
        self.stop()
        return (
            "None"
            if self.elapsed_time is None
            else string.pretty_duration(self.elapsed_time, **kwargs)
        )



from typing import Union, Any, List
import datetime
import numpy as np
from datetime import timezone
import math
import random as random_module
import string as string_module
import time

from blueness import module

from blue_options import NAME
from blue_options.string.constants import unit_of
from blue_options.logger import logger


NAME = module.name(__file__, NAME)


def after(
    s: str,
    sub_string: str,
    n: int = 1,
):
    if sub_string == "" or sub_string not in s:
        return ""

    return sub_string.join(s.split(sub_string)[n:])


def before(
    s: str,
    sub_string: str,
    n: int = 1,
):
    if sub_string == "" or sub_string not in s:
        return ""

    return sub_string.join(s.split(sub_string)[:n])


def between(
    s: str,
    sub_string_1: str,
    sub_string_2: str,
):
    return before(
        after(
            s,
            sub_string_1,
        ),
        sub_string_2,
    )


def pretty_bytes(
    byte_count: Union[None, int],
) -> str:
    if byte_count is None:
        return "Unknown"

    byte_count = round(byte_count)

    # Less than a kB
    if byte_count < 1024 / 10:
        return f"{byte_count:.0f} byte(s)"

    byte_count /= 1024

    if byte_count < 1024 / 10:
        return f"{byte_count:.2f} kB"

    byte_count /= 1024

    if byte_count < 1024:
        return f"{byte_count:.2f} MB"

    byte_count /= 1024

    if byte_count < 1024:
        return f"{byte_count:.2f} GB"

    byte_count /= 1024

    return f"{byte_count:.2f} TB"


def pretty_date(
    date: Any = None,
    as_filename: bool = False,
    delimiter: str = ", ",
    explicit_format: str = "",
    in_gmt: bool = False,
    include_date: bool = True,
    include_seconds: bool = True,
    include_time: bool = True,
    include_weekdays: bool = False,
    include_zone: bool = False,
    short: bool = False,
    squeeze: bool = False,
    unique: bool = False,
) -> str:
    format = ""
    if include_date:
        if short or as_filename:
            format = "%Y %m %d"
        else:
            format = "%d %B %Y"

        if include_weekdays:
            format = "%A, " + format

        if include_time:
            format += delimiter
    if include_time:
        format += "%H:%M"
        if include_seconds:
            format += ":%S"
    if unique:
        format += f":{random_module.randrange(100000):05d}"
    if as_filename:
        format = (
            format.replace(" ", "-")
            .replace(":", "-")
            .replace(".", "-")
            .replace(",", "")
        )

    if explicit_format:
        format = explicit_format

    if date is None:
        date = time.time()
    if isinstance(date, datetime.datetime):
        output = date.strftime(format)
    else:
        if in_gmt:
            output = time.strftime(format, time.gmtime(date))

            if include_zone:
                output += " (GMT)"
        else:
            output = time.strftime(format, time.localtime(date))

            if include_zone:
                output += f" ({time.tzname[0] if time.tzname else '?'})"

    if squeeze:
        output = output.replace("-", "")

    return output


def pretty_frequency(
    frequency: Union[None, float],
) -> str:
    if frequency is None or frequency == 0:
        return "None"

    if frequency >= 0.5:
        if frequency < 10**3:
            return f"{frequency:.1f} Hz"

        if frequency < 10**6:
            return f"{frequency / 10**3:.1f} kHz"

        if frequency < 10**6:
            return f"{frequency / 10**6:.1f} MHz"

        return f"{frequency / 10**9:.1f} GHz"

    return "1/{}".format(
        pretty_duration(
            1 / frequency,
            largest=True,
            short=True,
        )
    )


def pretty_shape(shape: List[int]) -> str:
    return "x".join([str(value) for value in list(shape)])


def pretty_shape_of_matrix(
    matrix: np.ndarray,
) -> str:
    return (
        f"{pretty_shape(matrix.shape)}:{matrix.dtype}"
        if isinstance(matrix, np.ndarray)
        else "-"
    )


def pretty_duration(
    duration: Union[None, int],
    element_format: str = "{}{}",
    include_ms: bool = False,
    largest: bool = False,
    past: bool = False,
    short: bool = False,
) -> str:
    if duration is None:
        return "None"

    negative_duration = duration < 0
    duration = abs(duration)

    duration_ = duration
    duration = math.floor(duration) if include_ms else round(duration)

    milliseconds = round(1000 * (duration_ - duration))

    seconds = duration % 60
    duration = math.floor(duration / 60)
    minutes = duration % 60
    duration = math.floor(duration / 60)
    hours = duration % 24
    duration = math.floor(duration / 24)
    days = duration % 30
    duration = math.floor(duration / 30)
    months = duration % 12
    years = math.floor(duration / 12)

    if short:
        year_name = " yr"
        month_name = " mth"
        day_name = " d"
        hour_name = " hr"
        minute_name = " min"
        second_name = " s"
        millisecond_name = " ms"
    else:
        year_name = " year(s)"
        month_name = " month(s)"
        day_name = " day(s)"
        hour_name = " hour(s)"
        minute_name = " minute(s)"
        second_name = " second(s)"
        millisecond_name = " millisecond(s)"

    output = []
    if years > 0:
        output.append(element_format.format(years, year_name))
    if months > 0:
        output.append(element_format.format(months, month_name))
    if days > 0:
        output.append(element_format.format(days, day_name))
    if hours > 0:
        output.append(element_format.format(hours, hour_name))
    if minutes > 0:
        output.append(element_format.format(minutes, minute_name))
    if seconds > 0:
        output.append(element_format.format(seconds, second_name))
    if include_ms:
        if milliseconds > 0:
            output.append("{:03d}{}".format(milliseconds, millisecond_name))

    if largest:
        output = output[:1]
    output = ", ".join(output)

    if past and output:
        output += " ago"

    if not output:
        output = "< 1{}".format(millisecond_name if include_ms else second_name)

    return ("-" if negative_duration else "") + output


def random(
    length: int = 16,
    alphabet: str = string_module.ascii_lowercase
    + string_module.digits
    + string_module.ascii_uppercase,
):
    return "".join(random_module.choice(alphabet) for _ in range(length))


def timestamp() -> str:
    return pretty_date(
        as_filename=True,
        include_time=True,
        unique=True,
    )


def utc_timestamp(
    date: Any = None,
    format: str = "%Y-%m-%d-%H-%M-%S",
    timezone_: str = "America/New_York",
) -> str:
    if date is None:
        # https://www.geeksforgeeks.org/get-utc-timestamp-in-python/
        return (
            datetime.datetime.now(timezone.utc).replace(tzinfo=timezone.utc).timestamp()
        )

    try:
        # https://stackoverflow.com/a/79877/17619982
        import pytz

        local = pytz.timezone(timezone_)
        naive = datetime.datetime.strptime(date, format)
        local_dt = local.localize(naive, is_dst=None)
        return local_dt.astimezone(pytz.utc).timestamp()
    except:
        logger.error(f"{NAME}.utc_timestamp({date},{format}): failed.")
        return "unknown"



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from blue_options import NAME
from blue_options.host import get_name
from blue_options.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    default="get",
    help="get",
)
parser.add_argument(
    "--keyword",
    type=str,
    help="name",
)
args = parser.parse_args()


success = False
# bash-tested in test_abcli_host in blue-objects.
if args.task == "get":
    success = True
    output = f"unknown-{args.keyword}"

    if args.keyword == "name":
        output = get_name()

    print(output)
else:
    success = None


sys_exit(logger, NAME, args.task, success, log=False)
